@@ -5,23 +5,27 @@ import datetime
 from os.path import exists	
 from openpyxl import Workbook
 from openpyxl import load_workbook
+from openpyxl.formatting.rule import ColorScaleRule
+from openpyxl.styles import PatternFill, Font
+from openpyxl.styles.differential import DifferentialStyle
+from openpyxl.formatting import Rule
 import os
 import argparse
 import mysql.connector
 import yaml
 from tqdm import tqdm
 
-#millores a fer:
-# apendre a anexar al excel per poder fer algunes funcions actualment commentades
+__version__ = "1.7.0"
 
-# Escriure o llegir del fitxer de config/config.yaml, en el qual es guarda la ultima data on es va agafar dades de synology menys un mes
+# Escriure o llegir del fitxer de config/config.yaml, en el qual es guarda la ultima data on es va agafar dades de synology
 # WoR determina si escriu "w" o si llegeix "r"
 # si es tria l'opcio de llegir retorna un string amb el temps en utc timestamp, sino no retorna res 
+# Es crida cuan es vol saber la ultima vegada desde quin punt s'han d'agafar les dades de les copies o per escriure el mateix
 def Data(WoR):
 	if WoR == "w":
 		with open("config/config.yaml") as f:
 			list_doc = yaml.safe_load(f)
-		list_doc[0]['data']=str(temps()-2592000)
+		list_doc[0]['data']=str(temps())
 		with open("config/config.yaml", 'w') as yamlfilew:
 			yaml.dump(list_doc, yamlfilew)
 			
@@ -85,7 +89,7 @@ def logout(url, sid, cookie):
 #Els parametres son la sid i la cookie per identificació i la url del NAS al cual recolectar les dades
 #Retorna les dades en format json i en cas de que dongui error retorna un text en format json sense dades per aixis evitar el issue #3 "Error en les dades que retorna despres de que es trobi amb un nas sense connexio"
 def InfoCopies(url, cookie, sid):#6 issue. A vegades dona error sense motiu aparent al fer-ho una segona vega es soluciona
-	copies_parameters = {"api":"SYNO.ActiveBackup.Overview", "version":"1", "method":"list_device_transfer_size", "time_start": int(Data("r")), "time_end": temps(), "_sid": sid}
+	copies_parameters = {"api":"SYNO.ActiveBackup.Overview", "version":"1", "method":"list_device_transfer_size", "time_start": int(Data("r"))-args.date, "time_end": temps(), "_sid": sid}
 	response = requests.get(url, params=copies_parameters, headers={"cookie":cookie}).json()
 	if	response['success'] == True:
 		if args.quiet:
@@ -262,28 +266,85 @@ def borrar(sheet, row):
 			return
 		sheet.delete_rows(row[0].row, 1)
 
+#Dona format i format condicional a la fulla
+#L'unica dada passada es la fulla
+#Nomes es crida cuan la fulla de excel no existia en la funcio prepExcel()
+def formatar(wsdefault, workbook):
+
+	#ESCALA DE COLORS
+	rule = ColorScaleRule(start_type='num', start_value=50, start_color='F8696B',
+				mid_type='num', mid_value=200, mid_color='FFEB84',
+				end_type='num', end_value=400, end_color='63BE7B'
+	)
+
+	#TEXT VERMELL
+	red_text = Font(color="9C0006")
+	red_fill = PatternFill(bgColor="FFC7CE")
+	dxf1 = DifferentialStyle(font=red_text, fill=red_fill)
+	rule1 = Rule(type="containsText", operator="containsText", text="ERROR", dxf=dxf1)
+	rule1.formula = ['NOT(ISERROR(SEARCH("ERROR",E1)))']
+
+	#TEXT GROC
+	yellow_text = Font(color="9C5700")
+	yellow_fill = PatternFill(bgColor="FFEB9C")
+	dxf2 = DifferentialStyle(font=yellow_text, fill=yellow_fill)
+	rule2 = Rule(type="containsText", operator="containsText", text="Warning", dxf=dxf2)
+	rule2.formula = ['NOT(ISERROR(SEARCH("Warning",E1)))']
+
+	#TEXT VERD
+	green_text = Font(color="006100")
+	green_fill = PatternFill(bgColor="C6EFCE")
+	dxf3 = DifferentialStyle(font=green_text, fill=green_fill)
+	rule3 = Rule(type="containsText", operator="containsText", text="Correcte", dxf=dxf3)
+	rule3.formula = ['NOT(ISERROR(SEARCH("Correcte",E1)))']
+
+	#TEXT TARONJA (Tamany MB)
+	orange_fill = PatternFill(bgColor="FFC000")
+	dxf4 = DifferentialStyle( fill=orange_fill)
+	rule4 = Rule(type="containsText", operator="beginsWith", text="0", dxf=dxf4)
+	rule3.formula = ['NOT(ISERROR(SEARCH("0",D1)))']
+
+	wsdefault.conditional_formatting.add('F1:F500', rule)
+	wsdefault.conditional_formatting.add('E1:E500', rule1)
+	wsdefault.conditional_formatting.add('E1:E500', rule2)
+	wsdefault.conditional_formatting.add('E1:E500', rule3)
+	wsdefault.conditional_formatting.add('D1:D500', rule4)
+
+	wsdefault.column_dimensions['A'].width = 19
+	wsdefault.column_dimensions['B'].width = 28
+	wsdefault.column_dimensions['C'].width = 10
+	wsdefault.column_dimensions['D'].width = 13
+	wsdefault.column_dimensions['E'].width = 8
+	wsdefault.column_dimensions['F'].width = 18
+	
+	workbook.save(fitxer)
+
 #Prepara el excel en cas de que no existis abans (borrant dades ateriors i posant la capçalera)
 #L'únic parametre es el document d'excel
 #No retorna res.
-def prepExcel(workbook):
+def prepExcel(workbook, existeix):
 	if args.quiet:
 		print("Preparant excel")
-	for sheet in workbook:
-		if sheet.title != "Sheet":
-			workbook.remove(sheet)
+	if existeix:
+		for sheet in workbook:
+			if sheet.title != "Sheet":
+				workbook.remove(sheet)
 			
-	wsdefault = workbook['Sheet']
-	
-	for row in wsdefault:
-		borrar(wsdefault,row)			
-
+		wsdefault = workbook['Sheet']
+		for row in wsdefault:
+			borrar(wsdefault,row)
+		workbook.save(fitxer)			
+	else:
+		wsdefault = workbook['Sheet']
+		formatar(wsdefault, workbook)
+		workbook.save(fitxer)
 	wsdefault.cell(row=1, column=1, value="Nom NAS")
 	wsdefault.cell(row=1, column=2, value="Nom Dispositiu")
 	wsdefault.cell(row=1, column=3, value="Data")
 	wsdefault.cell(row=1, column=4, value="Tamany MB")
 	wsdefault.cell(row=1, column=5, value="Status")
 	wsdefault.cell(row=1, column=6, value="Tamany Lliure GB")
-
+	workbook.save(fitxer)
 
 #Acces a la base de dades i recoleccio de la informacio
 #Els parametres son les credencials i la ip/host de la base de dades
@@ -347,107 +408,117 @@ def escriureDadesJSON(llistaFinal):
 	if not(args.quiet):
 		print("Done")
 
-###################################################################################################################################################################
+#Juntament amb les altres funcions agafa les dades de varis synologys les processa en un JSON i, si s'escull l'opcio un excel
+#No te paramatres pero te unes quantes variables globals
+def main():
+	parser = argparse.ArgumentParser(description='Una API per a recullir invormacio de varis NAS Synology que tinguin la versio 6 o mes.', epilog="Per configuracio adicional anar a config/config.yaml")
+	parser.add_argument('-e', '--excel', help='Guardar la informacio a un excel, per defecte esta desactivat', action="store_true")
+	parser.add_argument('-q', '--quiet', help='Nomes mostra els errors i el missatge de acabada per pantalla.', action="store_false")
+	parser.add_argument('-f', '--file', help='Especificar el fitxer de excel a on guardar. Per defecte es revisio_copies_seguretat_synology_vs1.xlsx', default="revisio_copies_seguretat_synology_vs1.xlsx", metavar="RUTA")
+	parser.add_argument('-d', '--date', type=int, help='La cantitat de temps (en segons) enrere que agafara les dades de copies. Per defecte es 2592000(un mes)', default=2592000, metavar='SEC')
+	parser.add_argument('-v', '--versio', help='Mostra la versio', action='version', version='Synology_API-NPP v1.7.0')
+	global args
+	args = parser.parse_args()
 
-parser = argparse.ArgumentParser(description='Una API per a recullir invormacio de varis NAS Synology que tinguin la versio 6 o mes.', epilog="Per configuracio adicional anar a config/config.yaml")
-parser.add_argument('-e', '--excel', help='Guardar la informacio a un excel, per defecte esta desactivat', action="store_true")
-parser.add_argument('-q', '--quiet', help='Nomes mostra els errors i el missatge de acabada per pantalla.', action="store_false")
-parser.add_argument('-f', '--file', help='Especificar el fitxer de excel a on guardar. Per defecte es revisio_copies_seguretat_synology_vs1.xlsx', default="revisio_copies_seguretat_synology_vs1.xlsx", metavar="RUTA")
-parser.add_argument('-v', '--versio', help='Mostra la versio', action='version', version='Synology_API-NPP vs1.6.8')
-args = parser.parse_args()
+	global current_transaction
+	current_transaction = 2
+	global fitxer
+	fitxer = args.file 
 
-current_transaction = 2
-fitxer = args.file 
+	if not(exists("config/config.yaml")):
+		print("Emplena el fitxer de configuracio de Base de Dades a config/config.yaml")
+		article_info = [
+			{
+				'BD': {
+				'host' : 'localhost',
+				'user': 'root',
+				'passwd': 'patata'
+				},
+				'data': str(temps())
+			}
+		]
+		with open("config/config.yaml", 'w') as yamlfile:
+			data = yaml.dump(article_info, yamlfile)
 
-if not(exists("config/config.yaml")):
-	print("Emplena el fitxer de configuracio de Base de Dades a config/config.yaml")
-	article_info = [
-    	{
-        	'BD': {
-    	    'host' : 'localhost',
-    	    'user': 'root',
-    	    'passwd': 'patata'
-    	    },
-			'data': str(temps()-2592000)
-    	}
-	]
-	with open("config/config.yaml", 'w') as yamlfile:
-		data = yaml.dump(article_info, yamlfile)
+	with open("config/config.yaml", "r") as yamlfile:
+		data = yaml.load(yamlfile, Loader=yaml.FullLoader)
 
-with open("config/config.yaml", "r") as yamlfile:
-    data = yaml.load(yamlfile, Loader=yaml.FullLoader)
+	servidor = data[0]['BD']['host']
+	usuari = data[0]['BD']['user']
+	contrassenya = data[0]['BD']['passwd']
 
-servidor = data[0]['BD']['host']
-usuari = data[0]['BD']['user']
-contrassenya = data[0]['BD']['passwd']
+	global taulabd
+	taulabd = bd(servidor, usuari, contrassenya)
 
-taulabd = bd(servidor, usuari, contrassenya)
-
-if exists(fitxer) == False:
-	workbook = Workbook()
-	prepExcel(workbook)
-	workbook.save(fitxer)
-elif args.excel:
-	workbook = load_workbook(filename = fitxer)
-	prepExcel(workbook)
-else:
-	workbook = load_workbook(filename = fitxer)
+	global workbook
+	if exists(fitxer) == False:
+		workbook = Workbook()
+		prepExcel(workbook, False)
+		workbook.save(fitxer)
+	elif args.excel:
+		try:
+			workbook = load_workbook(filename = fitxer)
+			prepExcel(workbook, True)
+		except:
+			print("Tanca el excel i torna-ho a provar")
+	else:
+		workbook = load_workbook(filename = fitxer)
 
 
-llistaTransf = []
-llistadispCopia = []
-llistaNAS = []
-dadesCopiesTotes = recoleccioDades(workbook)
-num_nas = len(dadesCopiesTotes)
-
-# y es cada transaccio (es reseteja per cada dispositiu)
-# z es personalitzat que es per cada dispositiu que tingui transaccio (es reseteja per cada NAS)
-# x es cada dispositiu (es reseteja per cada NAS)
-# nas es cada nas (es reseteja cada execucio)
-# current_transaction es cada transaccio (es reseteja cada execucio)
-nas = 0
-nom_dispositiu=""
-for nas in tqdm (range(num_nas), desc="Processar Dades", ncols=100, disable=args.quiet):
-	nom_nas = taulabd[nas][0]
-	id_pandora = taulabd[nas][5]
-	num_copies = int(dadesCopiesTotes[nas]['data']['total'])
-	tamanyLliure=tamanyRestant(nas)
-	x = 0
-	z = 0
-	while x < num_copies: 
-		if len(dadesCopiesTotes[nas]['data']['device_list'][x]['transfer_list']) != 0:
-			num_transferencies = len(dadesCopiesTotes[nas]['data']['device_list'][x]['transfer_list'])
-
-			y=0
-			for y in tqdm (range (num_transferencies), desc=nom_nas +" | "+ dadesCopiesTotes[nas]['data']['device_list'][x]['transfer_list'][y]['device_name'], ncols=75, disable=not(args.quiet)):
-				nom_dispositiu = dadesCopiesTotes[nas]['data']['device_list'][x]['transfer_list'][y]['device_name']
-				status = statusConvertor(dadesCopiesTotes[nas]['data']['device_list'][x]['transfer_list'][y]['status'])
-				tamany_transferencia = dadesCopiesTotes[nas]['data']['device_list'][x]['transfer_list'][y]['transfered_bytes']
-				temps_finalitzacio = dadesCopiesTotes[nas]['data']['device_list'][x]['transfer_list'][y]['time_end']
-
-				file_time = datetime.datetime.fromtimestamp(temps_finalitzacio)
-				dataF=file_time.strftime('%Y-%m-%d')
-				if y+1 < num_transferencies:
-					u=1 #no se que fa
-				else:
-					llistaTransf.append({"data":dataF, "status":status, "tamany_transferenciaMB":(tamany_transferencia/1024)/1024})
-				if args.excel:
-					escriptorExcel(nom_dispositiu, status, temps_finalitzacio, tamany_transferencia, workbook, y, z, nom_nas, tamanyLliure)
-					try:
-						workbook.save(fitxer)
-					except:
-						now = datetime.datetime.now()
-						date_string = now.strftime('%Y-%m-%d--%H-%M-%S-permisos')
-						f = open("errorLogs/"+date_string+".txt",'w')
-						f.write("Error de permisos en obrir el Excel (Pot ser que el excel estigui obert?)")
-						f.close()
-						print("Error de permisos")
-			z += 1
-		llistadispCopia.append({"nomDispositiu":nom_dispositiu, "Transferencies":llistaTransf})
-		nom_dispositiu = ""
-		llistaTransf = []
-		x += 1
-	llistaNAS.append({"nomNAS":nom_nas, "ID Pandora":id_pandora, "copies":llistadispCopia})
+	llistaTransf = []
 	llistadispCopia = []
-	nas +=1
-escriureDadesJSON([{"NAS":llistaNAS}])
\ No newline at end of file
+	llistaNAS = []
+	dadesCopiesTotes = recoleccioDades(workbook)
+	num_nas = len(dadesCopiesTotes)
+
+	# y es cada transaccio (es reseteja per cada dispositiu)
+	# z es personalitzat que es per cada dispositiu que tingui transaccio (es reseteja per cada NAS)
+	# x es cada dispositiu (es reseteja per cada NAS)
+	# nas es cada nas (es reseteja cada execucio)
+	# current_transaction es cada transaccio (es reseteja cada execucio)
+	nas = 0
+	nom_dispositiu=""
+	for nas in tqdm (range(num_nas), desc="Processar Dades", disable=args.quiet):
+		nom_nas = taulabd[nas][0]
+		id_pandora = taulabd[nas][5]
+		num_copies = int(dadesCopiesTotes[nas]['data']['total'])
+		tamanyLliure=tamanyRestant(nas)
+		x = 0
+		z = 0
+		while x < num_copies: 
+			if len(dadesCopiesTotes[nas]['data']['device_list'][x]['transfer_list']) != 0:
+				num_transferencies = len(dadesCopiesTotes[nas]['data']['device_list'][x]['transfer_list'])
+
+				y=0
+				for y in tqdm (range (num_transferencies), desc=nom_nas +" | "+ dadesCopiesTotes[nas]['data']['device_list'][x]['transfer_list'][y]['device_name'], disable=not(args.quiet)):
+					nom_dispositiu = dadesCopiesTotes[nas]['data']['device_list'][x]['transfer_list'][y]['device_name']
+					status = statusConvertor(dadesCopiesTotes[nas]['data']['device_list'][x]['transfer_list'][y]['status'])
+					tamany_transferencia = dadesCopiesTotes[nas]['data']['device_list'][x]['transfer_list'][y]['transfered_bytes']
+					temps_finalitzacio = dadesCopiesTotes[nas]['data']['device_list'][x]['transfer_list'][y]['time_end']
+
+					file_time = datetime.datetime.fromtimestamp(temps_finalitzacio)
+					dataF=file_time.strftime('%Y-%m-%d')
+					if y+1 < num_transferencies:
+						u=1 #no se que fa
+					else:
+						llistaTransf.append({"data":dataF, "status":status, "tamany_transferenciaMB":(tamany_transferencia/1024)/1024})
+					if args.excel:
+						escriptorExcel(nom_dispositiu, status, temps_finalitzacio, tamany_transferencia, workbook, y, z, nom_nas, tamanyLliure)
+						try:
+							workbook.save(fitxer)
+						except:
+							now = datetime.datetime.now()
+							date_string = now.strftime('%Y-%m-%d--%H-%M-%S-permisos')
+							f = open("errorLogs/"+date_string+".txt",'w')
+							f.write("Error de permisos en obrir el Excel (Pot ser que el excel estigui obert?)")
+							f.close()
+							print("Error de permisos")
+				z += 1
+			llistadispCopia.append({"nomDispositiu":nom_dispositiu, "Transferencies":llistaTransf})
+			nom_dispositiu = ""
+			llistaTransf = []
+			x += 1
+		llistaNAS.append({"nomNAS":nom_nas, "ID Pandora":id_pandora, "copies":llistadispCopia})
+		llistadispCopia = []
+		nas +=1
+	escriureDadesJSON([{"NAS":llistaNAS}])
